#!/usr/bin/env python
#Copyright (c) 2011 Isaac Dawson (WBTS Project)
#Permission is hereby granted, free of charge, to any person obtaining a copy 
#of this software and associated documentation files (the "Software"), to deal 
#in the Software without restriction, including without limitation the rights 
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
#copies of the Software, and to permit persons to whom the Software is furnished
#to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in all
#copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
#THE SOFTWARE.

import os
from signal import SIGTERM
from twisted.internet import protocol
from twisted.internet import reactor

class BrowserProcess(protocol.ProcessProtocol):
    """Handles executing the browser process via twisted. Access the .pid if
    you need to attach a debugger via PID.
    
    @ivar caller: The caller service to call back into on start/finish. Must
    have process_started and process_finished methods bound to it.
    """
    def __init__(self, caller):
        self.caller = caller
        self.data = ''
    
    def killProcess(self):
        print "[*] Killing Browser Process"
        os.kill(self.pid, SIGTERM)
    
    def connectionMade(self):
        self.pid = self.transport.pid
        print "[*] Browser Process Started"
        self.caller.process_started()

    def outReceived(self, data):
        print "[*] Browser (stdout): %s"%(data,)
    
    def errReceived(self, data):
        print "[*] Browser (stderr): %s"%(data,)
       
    def processEnded(self, reason):
        print "[*] Browser Process Started"
        self.caller.process_finished()

